!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center
! Land Information System Framework (LISF)
! Version 7.4
!
! Copyright (c) 2022 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
!
! !ROUTINE: AGRMET_calcrh_dpt
! \label{AGRMET_calcrh_dpt}
!
! !REVISION HISTORY:
! 
!     10 apr 89  initial version.........................msgt neill/sddc
!     11 feb 91  chgd the floor rh value to 0.1 (10%). also isolated 
!                the debug stmt and declared it, itf, and k
!                ..........................................mr moore/sddc
!     04 mar 99  chgd the floor rh value back to 0.0 and increased the
!                scope of the 2nd element of the array 'v' to -100:100
!                to match the rhbds block data stmt........mr moore/dnxm
!     14 jul 99  ported to IBM SP-2.  pass in vapor pressure values
!                thru the rhbds include file instead of thru a
!                common block..............................mr gayno/dnxm
!     20 aug 05 Sujay Kumar, Initial Specification in LIS 
!     17 may 10  Duplicated AGRMET_calcrh but accepts dew point rather
!                than temp/dew point depression to save cycles
!                .............................Chris Franks 16WS/WXE/SEMS
! 
! !INTERFACE: 
function AGRMET_calcrh_dpt ( temp, dpt )
  implicit none
! !ARGUMENTS: 
  real,   intent(in)         :: temp
  real,   intent(in)         :: dpt
!
! !DESCRIPTION: 
!     to calculate relative humidities given temperatures and dew point
!     temperature. 
! 
!     \subsubsection{Method}
!     - convert temp \& dew point to celsius
!     - if temps are within reasonable range
!       - derive look-up table indices for getting ambient and 
!         saturation vapor pressures,  then get them. 
!       - calc rh by dividing ambient vapor pressure by saturation vapor
!         pressure. keep value btwn 0 and 1
!     - if temps are not in reasonable range, set rh to -1
! 
!    The arguments are: 
!    \begin{description}
!     \item[temp]
!      temperature
!     \item[dpt]
!      dew point temperature 
!     \item[AGRMET\_calcrh\_dpt]
!      relative humidity
!     \end{description}
!EOP

  integer                    :: it
  integer                    :: itf
  integer                    :: k
  
  real                       :: AGRMET_calcrh_dpt
  real                       :: e(2)
  real                       :: t(2)
  

  integer                   :: i
  
  real                      :: v  ( 0:9,-100:100 )  
  

  data (v(i,-100),i=0,9) &
       /  .32806e-02,  .30174e-02,  .30212e-02,  .29984e-02,  .28152e-02,&
       .28572e-02,  .28534e-02,  .27771e-02,  .26894e-02,  .27161e-02/
  
  data (v(i, -99),i=0,9) &
       /  .28877e-02,  .27237e-02,  .27351e-02,  .26550e-02,  .28725e-02, &
       .28000e-02,  .28305e-02,  .29793e-02,  .29144e-02,  .29755e-02/

  data (v(i, -98),i=0,9) &
       /  .30861e-02,  .31052e-02,  .31891e-02,  .32845e-02,  .34256e-02, &
       .33760e-02,  .34904e-02,  .36545e-02,  .36850e-02,  .38910e-02/

  data (v(i, -97),i=0,9) &
       /  .39253e-02,  .40894e-02,  .41161e-02,  .42076e-02,  .43488e-02, &
       .45013e-02,  .46120e-02,  .48294e-02,  .49667e-02,  .50163e-02/
  
  data (v(i, -96),i=0,9) &
       /  .51765e-02,  .52948e-02,  .53749e-02,  .56190e-02,  .57220e-02, &
       .58403e-02,  .60425e-02,  .61798e-02,  .65002e-02,  .66032e-02/

  data (v(i, -95),i=0,9) &
       /  .67635e-02,  .69542e-02,  .71449e-02,  .73051e-02,  .75378e-02, &
       .76637e-02,  .78392e-02,  .80070e-02,  .82550e-02,  .84534e-02/

  data (v(i, -94),i=0,9) &
       /  .86784e-02,  .88120e-02,  .90675e-02,  .92583e-02,  .95139e-02,&
       .96321e-02,  .98114e-02,  .10151e-01,  .10334e-01,  .10468e-01/
  
  data (v(i, -93),i=0,9) &
       /  .10815e-01,  .10990e-01,  .11219e-01,  .11395e-01,  .11738e-01, &
       .11868e-01,  .12184e-01,  .12451e-01,  .12691e-01,  .12981e-01/
  
  data (v(i, -92),i=0,9) &
       /  .13233e-01,  .13489e-01,  .13672e-01,  .13977e-01,  .14309e-01, &
       .14530e-01,  .14755e-01,  .15167e-01,  .15415e-01,  .15640e-01/

  data (v(i, -91),i=0,9) &
       /  .15999e-01,  .16308e-01,  .16563e-01,  .16891e-01,  .17284e-01, &
       .17563e-01,  .17838e-01,  .18188e-01,  .18524e-01,  .18829e-01/

  data (v(i, -90),i=0,9) &
       /  .19176e-01,  .19581e-01,  .19928e-01,  .20214e-01,  .20668e-01,&
       .21008e-01,  .21358e-01,  .21744e-01,  .22148e-01,  .22503e-01/

  data (v(i, -89),i=0,9) &
       /  .22999e-01,  .23319e-01,  .23731e-01,  .24143e-01,  .24567e-01, &
       .24986e-01,  .25482e-01,  .25940e-01,  .26379e-01,  .26871e-01/
  
  data (v(i, -88),i=0,9) &
       /  .27275e-01,  .27710e-01,  .28282e-01,  .28767e-01,  .29266e-01, &
       .29800e-01,  .30289e-01,  .30865e-01,  .31315e-01,  .31868e-01/

  data (v(i, -87),i=0,9) &
       /  .32455e-01,  .33016e-01,  .33543e-01,  .34111e-01,  .34760e-01, &
       .35362e-01,  .35984e-01,  .36591e-01,  .37285e-01,  .37937e-01/

  data (v(i, -86),i=0,9) &
       /  .38628e-01,  .39192e-01,  .39932e-01,  .40657e-01,  .41363e-01, &
       .42027e-01,  .42751e-01,  .43491e-01,  .44243e-01,  .45036e-01/
  
  data (v(i, -85),i=0,9) &
       /  .45811e-01,  .46589e-01,  .47424e-01,  .48233e-01,  .49049e-01, &
       .49911e-01,  .50797e-01,  .51605e-01,  .52528e-01,  .53467e-01/

  data (v(i, -84),i=0,9) &
       /  .54329e-01,  .55267e-01,  .56267e-01,  .57167e-01,  .58105e-01, &
       .59135e-01,  .60181e-01,  .61195e-01,  .62237e-01,  .63297e-01/

  data (v(i, -83),i=0,9) &
       /  .64396e-01,  .65536e-01,  .66612e-01,  .67776e-01,  .68867e-01, &
       .70042e-01,  .71243e-01,  .72468e-01,  .73704e-01,  .75016e-01/

  data (v(i, -82),i=0,9) &
       /  .76218e-01,  .77511e-01,  .78873e-01,  .80173e-01,  .81558e-01, &
       .82916e-01,  .84316e-01,  .85690e-01,  .87193e-01,  .88623e-01/

  data (v(i, -81),i=0,9) &
       /  .90137e-01,  .91618e-01,  .93182e-01,  .94742e-01,  .96256e-01, &
       .97942e-01,  .99529e-01,  .10120e+00,  .10290e+00,  .10458e+00/

  data (v(i, -80),i=0,9) &
       /  .10634e+00,  .10807e+00,  .10984e+00,  .11166e+00,  .11349e+00, &
       .11542e+00,  .11730e+00,  .11921e+00,  .12118e+00,  .12319e+00/

  data (v(i, -79),i=0,9) &
       /  .12521e+00,  .12721e+00,  .12933e+00,  .13144e+00,  .13362e+00, &
       .13573e+00,  .13794e+00,  .14018e+00,  .14238e+00,  .14477e+00/

  data (v(i, -78),i=0,9) &
       /  .14709e+00,  .14947e+00,  .15184e+00,  .15429e+00,  .15679e+00, &
       .15927e+00,  .16182e+00,  .16443e+00,  .16704e+00,  .16965e+00/
  
  data (v(i, -77),i=0,9) &
       /  .17237e+00,  .17513e+00,  .17795e+00,  .18070e+00,  .18361e+00, &
       .18650e+00,  .18942e+00,  .19244e+00,  .19542e+00,  .19850e+00/
  
  data (v(i, -76),i=0,9) &
       /  .20160e+00,  .20475e+00,  .20795e+00,  .21116e+00,  .21453e+00, &
       .21778e+00,  .22121e+00,  .22467e+00,  .22809e+00,  .23163e+00/
  
  data (v(i, -75),i=0,9) &
       /  .23524e+00,  .23889e+00,  .24253e+00,  .24627e+00,  .25005e+00, &
       .25385e+00,  .25774e+00,  .26171e+00,  .26564e+00,  .26970e+00/
  
  data (v(i, -74),i=0,9) &
       /  .27383e+00,  .27797e+00,  .28220e+00,  .28647e+00,  .29080e+00, &
       .29518e+00,  .29963e+00,  .30412e+00,  .30870e+00,  .31334e+00/
        
  data (v(i, -73),i=0,9) &
       /  .31807e+00,  .32279e+00,  .32764e+00,  .33251e+00,  .33749e+00, &
       .34248e+00,  .34760e+00,  .35275e+00,  .35800e+00,  .36326e+00/

  data (v(i, -72),i=0,9) &
       /  .36863e+00,  .37403e+00,  .37960e+00,  .38517e+00,  .39084e+00, &
       .39659e+00,  .40239e+00,  .40829e+00,  .41423e+00,  .42029e+00/

  data (v(i, -71),i=0,9) &
       /  .42638e+00,  .43260e+00,  .43888e+00,  .44529e+00,  .45173e+00, &
       .45828e+00,  .46489e+00,  .47161e+00,  .47844e+00,  .48530e+00/

  data (v(i, -70),i=0,9) &
       /  .49229e+00,  .49937e+00,  .50653e+00,  .51377e+00,  .52111e+00, &
       .52858e+00,  .53613e+00,  .54377e+00,  .55149e+00,  .55937e+00/

  data (v(i, -69),i=0,9) &
       /  .56731e+00,  .57536e+00,  .58349e+00,  .59175e+00,  .60012e+00, &
       .60860e+00,  .61715e+00,  .62583e+00,  .63464e+00,  .64355e+00/
  
  data (v(i, -68),i=0,9) &
       /  .65259e+00,  .66177e+00,  .67099e+00,  .68037e+00,  .68987e+00, &
       .69951e+00,  .70921e+00,  .71907e+00,  .72908e+00,  .73921e+00/

  data (v(i, -67),i=0,9) &
       /  .74946e+00,  .75985e+00,  .77034e+00,  .78098e+00,  .79177e+00, &
       .80270e+00,  .81371e+00,  .82491e+00,  .83625e+00,  .84774e+00/
  
  data (v(i, -66),i=0,9) &
       /  .85935e+00,  .87111e+00,  .88302e+00,  .89508e+00,  .90728e+00, &
       .91965e+00,  .93219e+00,  .94484e+00,  .95767e+00,  .97067e+00/

  data (v(i, -65),i=0,9) &
       /  .98382e+00,  .99713e+00,  .10106e+01,  .10243e+01,  .10381e+01, &
       .10521e+01,  .10662e+01,  .10806e+01,  .10951e+01,  .11098e+01/

  data (v(i, -64),i=0,9) &
       /  .11247e+01,  .11397e+01,  .11550e+01,  .11704e+01,  .11860e+01, &
       .12018e+01,  .12179e+01,  .12341e+01,  .12505e+01,  .12671e+01/
  
  data (v(i, -63),i=0,9) &
       /  .12839e+01,  .13009e+01,  .13181e+01,  .13355e+01,  .13532e+01, &
       .13710e+01,  .13891e+01,  .14073e+01,  .14259e+01,  .14446e+01/
  
  data (v(i, -62),i=0,9) &
       /  .14636e+01,  .14827e+01,  .15022e+01,  .15218e+01,  .15417e+01, &
       .15618e+01,  .15822e+01,  .16028e+01,  .16237e+01,  .16448e+01/

  data (v(i, -61),i=0,9) &
       /  .16661e+01,  .16878e+01,  .17096e+01,  .17317e+01,  .17542e+01, &
       .17768e+01,  .17997e+01,  .18230e+01,  .18464e+01,  .18702e+01/

  data (v(i, -60),i=0,9) &
       /  .18943e+01,  .19186e+01,  .19431e+01,  .19681e+01,  .19932e+01, &
       .20187e+01,  .20445e+01,  .20706e+01,  .20970e+01,  .21237e+01/

  data (v(i, -59),i=0,9) &
       /  .21507e+01,  .21780e+01,  .22057e+01,  .22337e+01,  .22619e+01, &
       .22906e+01,  .23195e+01,  .23488e+01,  .23784e+01,  .24084e+01/

  data (v(i, -58),i=0,9) &
       /  .24387e+01,  .24694e+01,  .25004e+01,  .25318e+01,  .25635e+01, &
       .25956e+01,  .26281e+01,  .26609e+01,  .26941e+01,  .27277e+01/

  data (v(i, -57),i=0,9) &
       /  .27617e+01,  .27961e+01,  .28308e+01,  .28660e+01,  .29016e+01, &
       .29375e+01,  .29739e+01,  .30106e+01,  .30478e+01,  .30854e+01/

  data (v(i, -56),i=0,9) &
       /  .31235e+01,  .31620e+01,  .32008e+01,  .32402e+01,  .32799e+01, &
       .33201e+01,  .33608e+01,  .34019e+01,  .34435e+01,  .34855e+01/

  data (v(i, -55),i=0,9) &
       /  .35281e+01,  .35710e+01,  .36145e+01,  .36584e+01,  .37029e+01, &
       .37478e+01,  .37932e+01,  .38391e+01,  .38855e+01,  .39325e+01/

  data (v(i, -54),i=0,9) &
       /  .39799e+01,  .40279e+01,  .40764e+01,  .41254e+01,  .41750e+01, &
       .42251e+01,  .42757e+01,  .43269e+01,  .43787e+01,  .44310e+01/

  data (v(i, -53),i=0,9) &
       /  .44839e+01,  .45373e+01,  .45914e+01,  .46460e+01,  .47012e+01, &
       .47570e+01,  .48134e+01,  .48704e+01,  .49280e+01,  .49863e+01/
  
  data (v(i, -52),i=0,9) &
       /  .50451e+01,  .51046e+01,  .51647e+01,  .52255e+01,  .52869e+01, &
       .53490e+01,  .54117e+01,  .54751e+01,  .55392e+01,  .56039e+01/

  data (v(i, -51),i=0,9) &
       /  .56693e+01,  .57354e+01,  .58022e+01,  .58698e+01,  .59380e+01, &
       .60069e+01,  .60766e+01,  .61469e+01,  .62181e+01,  .62899e+01/
  
  data (v(i, -50),i=0,9) &
       /  .63625e+01,  .64144e+01,  .64919e+01,  .65701e+01,  .66490e+01, &
       .67286e+01,  .68089e+01,  .68899e+01,  .69716e+01,  .70540e+01/
  
  data (v(i, -49),i=0,9) &
       /  .71372e+01,  .72210e+01,  .73056e+01,  .73910e+01,  .74772e+01, &
       .75641e+01,  .76518e+01,  .77403e+01,  .78296e+01,  .79197e+01/
  
  data (v(i, -48),i=0,9) &
       /  .80106e+01,  .81024e+01,  .81950e+01,  .82885e+01,  .83828e+01, &
       .84780e+01,  .85740e+01,  .86710e+01,  .87688e+01,  .88676e+01/
  
  data (v(i, -47),i=0,9) &
       /  .89672e+01,  .90678e+01,  .91694e+01,  .92719e+01,  .93754e+01, &
       .94798e+01,  .95852e+01,  .96916e+01,  .97990e+01,  .99074e+01/
  
  data (v(i, -46),i=0,9) &
       /  .10017e+02,  .10127e+02,  .10239e+02,  .10351e+02,  .10465e+02, &
       .10580e+02,  .10696e+02,  .10813e+02,  .10931e+02,  .11050e+02/

  data (v(i, -45),i=0,9) &
       /  .11170e+02,  .11292e+02,  .11414e+02,  .11538e+02,  .11663e+02, &
       .11789e+02,  .11916e+02,  .12045e+02,  .12175e+02,  .12306e+02/
  
  data (v(i, -44),i=0,9) &
       /  .12438e+02,  .12572e+02,  .12707e+02,  .12843e+02,  .12980e+02, &
       .13119e+02,  .13259e+02,  .13401e+02,  .13544e+02,  .13688e+02/

  data (v(i, -43),i=0,9) &
       /  .13834e+02,  .13981e+02,  .14129e+02,  .14279e+02,  .14430e+02, &
       .14583e+02,  .14737e+02,  .14893e+02,  .15050e+02,  .15209e+02/
  
  data (v(i, -42),i=0,9) &
       /  .15369e+02,  .15530e+02,  .15694e+02,  .15859e+02,  .16025e+02, &
       .16193e+02,  .16363e+02,  .16534e+02,  .16707e+02,  .16881e+02/
  
  data (v(i, -41),i=0,9) &
       /  .17058e+02,  .17236e+02,  .17415e+02,  .17596e+02,  .17779e+02, &
       .17964e+02,  .18151e+02,  .18339e+02,  .18529e+02,  .18721e+02/

  data (v(i, -40),i=0,9) &
       /  .18915e+02,  .19111e+02,  .19308e+02,  .19507e+02,  .19708e+02, &
       .19912e+02,  .20117e+02,  .20324e+02,  .20533e+02,  .20743e+02/

  data (v(i, -39),i=0,9) &
       /  .20956e+02,  .21171e+02,  .21388e+02,  .21607e+02,  .21828e+02, &
       .22051e+02,  .22276e+02,  .22504e+02,  .22733e+02,  .22965e+02/

  data (v(i, -38),i=0,9) &
       /  .23199e+02,  .23435e+02,  .23673e+02,  .23913e+02,  .24156e+02, &
       .24400e+02,  .24648e+02,  .24897e+02,  .25149e+02,  .25403e+02/

  data (v(i, -37),i=0,9) &
       /  .25659e+02,  .25918e+02,  .26179e+02,  .26443e+02,  .26709e+02, &
       .26978e+02,  .27249e+02,  .27522e+02,  .27798e+02,  .28077e+02/
  
  data (v(i, -36),i=0,9) &
       /  .28358e+02,  .28642e+02,  .28928e+02,  .29217e+02,  .29508e+02, &
       .29803e+02,  .30100e+02,  .30399e+02,  .30701e+02,  .31007e+02/

  data (v(i, -35),i=0,9) &
       /  .31314e+02,  .31625e+02,  .31939e+02,  .32255e+02,  .32574e+02, &
       .32896e+02,  .33221e+02,  .33549e+02,  .33880e+02,  .34214e+02/
  
  data (v(i, -34),i=0,9) &
       /  .34551e+02,  .34890e+02,  .35233e+02,  .35579e+02,  .35928e+02, &
       .36281e+02,  .36636e+02,  .36995e+02,  .37356e+02,  .37721e+02/

  data (v(i, -33),i=0,9) &
       /  .38089e+02,  .38461e+02,  .38836e+02,  .39214e+02,  .39595e+02, &
       .39980e+02,  .40368e+02,  .40760e+02,  .41155e+02,  .41554e+02/

  data (v(i, -32),i=0,9) &
       /  .41956e+02,  .42361e+02,  .42770e+02,  .43183e+02,  .43600e+02, &
       .44020e+02,  .44443e+02,  .44871e+02,  .45302e+02,  .45737e+02/
  
  data (v(i, -31),i=0,9) &
       /  .46175e+02,  .46618e+02,  .47064e+02,  .47514e+02,  .47968e+02, &
       .48426e+02,  .48888e+02,  .49354e+02,  .49824e+02,  .50299e+02/
  
  data (v(i, -30),i=0,9) &
       /  .50777e+02,  .51259e+02,  .51745e+02,  .52236e+02,  .52731e+02, &
       .53230e+02,  .53733e+02,  .54240e+02,  .54752e+02,  .55269e+02/
  
  data (v(i, -29),i=0,9) &
       /  .55789e+02,  .56314e+02,  .56844e+02,  .57378e+02,  .57917e+02, &
       .58460e+02,  .59007e+02,  .59560e+02,  .60117e+02,  .60679e+02/

  data (v(i, -28),i=0,9) &
       /  .61245e+02,  .61816e+02,  .62392e+02,  .62973e+02,  .63559e+02, &
       .64149e+02,  .64745e+02,  .65346e+02,  .65951e+02,  .66562e+02/
  
  data (v(i, -27),i=0,9) &
       /  .67178e+02,  .67798e+02,  .68424e+02,  .69056e+02,  .69692e+02, &
       .70334e+02,  .70981e+02,  .71633e+02,  .72291e+02,  .72954e+02/
  
  data (v(i, -26),i=0,9) &
       /  .73623e+02,  .74297e+02,  .74977e+02,  .75662e+02,  .76353e+02, &
       .77050e+02,  .77752e+02,  .78460e+02,  .79174e+02,  .79894e+02/
  
  data (v(i, -25),i=0,9) &
       /  .80620e+02,  .81351e+02,  .82089e+02,  .82832e+02,  .83582e+02, &
       .84337e+02,  .85099e+02,  .85867e+02,  .86641e+02,  .87421e+02/
  
  data (v(i, -24),i=0,9) &
       /  .88208e+02,  .89001e+02,  .89801e+02,  .90606e+02,  .91419e+02, &
       .92238e+02,  .93063e+02,  .93895e+02,  .94734e+02,  .95579e+02/

  data (v(i, -23),i=0,9) &
       /  .96432e+02,  .97291e+02,  .98157e+02,  .99029e+02,  .99909e+02, &
       .10080e+03,  .10169e+03,  .10259e+03,  .10350e+03,  .10441e+03/
  
  data (v(i, -22),i=0,9) &
       /  .10534e+03,  .10627e+03,  .10720e+03,  .10815e+03,  .10910e+03, &
       .11006e+03,  .11103e+03,  .11200e+03,  .11298e+03,  .11397e+03/
  
  data (v(i, -21),i=0,9) &
       /  .11497e+03,  .11598e+03,  .11699e+03,  .11801e+03,  .11904e+03, &
       .12008e+03,  .12112e+03,  .12218e+03,  .12324e+03,  .12431e+03/
  
  data (v(i, -20),i=0,9) &
       /  .12539e+03,  .12647e+03,  .12757e+03,  .12867e+03,  .12978e+03, &
       .13090e+03,  .13203e+03,  .13317e+03,  .13432e+03,  .13547e+03/

  data (v(i, -19),i=0,9) &
       /  .13664e+03,  .13781e+03,  .13899e+03,  .14019e+03,  .14139e+03, &
       .14260e+03,  .14382e+03,  .14504e+03,  .14628e+03,  .14753e+03/
  
  data (v(i, -18),i=0,9) &
       /  .14879e+03,  .15005e+03,  .15133e+03,  .15261e+03,  .15391e+03, &
       .15521e+03,  .15653e+03,  .15785e+03,  .15919e+03,  .16053e+03/
  
  data (v(i, -17),i=0,9) &
       /  .16189e+03,  .16325e+03,  .16463e+03,  .16602e+03,  .16741e+03, &
       .16882e+03,  .17024e+03,  .17166e+03,  .17310e+03,  .17455e+03/
  
  data (v(i, -16),i=0,9) &
       /  .17601e+03,  .17748e+03,  .17897e+03,  .18046e+03,  .18196e+03, &
       .18348e+03,  .18501e+03,  .18654e+03,  .18809e+03,  .18965e+03/
  
  data (v(i, -15),i=0,9) &
       /  .19123e+03,  .19281e+03,  .19441e+03,  .19601e+03,  .19763e+03, &
       .19927e+03,  .20091e+03,  .20256e+03,  .20423e+03,  .20591e+03/
  
  data (v(i, -14),i=0,9) &
       /  .20760e+03,  .20931e+03,  .21103e+03,  .21276e+03,  .21450e+03, &
       .21625e+03,  .21802e+03,  .21980e+03,  .22160e+03,  .22340e+03/

  data (v(i, -13),i=0,9) &
       /  .22522e+03,  .22705e+03,  .22890e+03,  .23076e+03,  .23263e+03, &
       .23452e+03,  .23642e+03,  .23834e+03,  .24026e+03,  .24220e+03/
  
  data (v(i, -12),i=0,9) &
       /  .24416e+03,  .24613e+03,  .24811e+03,  .25011e+03,  .25213e+03, &
        .25415e+03,  .25619e+03,  .25825e+03,  .26032e+03,  .26241e+03/
  
  data (v(i, -11),i=0,9) &
       /  .26451e+03,  .26662e+03,  .26875e+03,  .27090e+03,  .27306e+03, &
       .27524e+03,  .27743e+03,  .27963e+03,  .28186e+03,  .28410e+03/

  data (v(i, -10),i=0,9) &
       /  .28635e+03,  .28862e+03,  .29091e+03,  .29321e+03,  .29553e+03, &
       .29786e+03,  .30021e+03,  .30258e+03,  .30497e+03,  .30737e+03/

  data (v(i,  -9),i=0,9) &
       /  .30979e+03,  .31222e+03,  .31467e+03,  .31714e+03,  .31963e+03, &
       .32213e+03,  .32465e+03,  .32719e+03,  .32975e+03,  .33232e+03/
  
  data (v(i,  -8),i=0,9) &
       /  .33492e+03,  .33753e+03,  .34015e+03,  .34280e+03,  .34547e+03, &
       .34815e+03,  .35085e+03,  .35357e+03,  .35631e+03,  .35907e+03/

  data (v(i,  -7),i=0,9) &
       /  .36184e+03,  .36464e+03,  .36746e+03,  .37029e+03,  .37314e+03, &
       .37602e+03,  .37891e+03,  .38182e+03,  .38476e+03,  .38771e+03/

  data (v(i,  -6),i=0,9) &
       /  .39068e+03,  .39368e+03,  .39669e+03,  .39972e+03,  .40278e+03, &
       .40585e+03,  .40895e+03,  .41207e+03,  .41521e+03,  .41837e+03/

  data (v(i,  -5),i=0,9) &
       /  .42155e+03,  .42475e+03,  .42797e+03,  .43122e+03,  .43449e+03, &
       .43778e+03,  .44109e+03,  .44442e+03,  .44778e+03,  .45116e+03/
  
  data (v(i,  -4),i=0,9) &
       /  .45456e+03,  .45799e+03,  .46143e+03,  .46490e+03,  .46840e+03, &
       .47191e+03,  .47545e+03,  .47902e+03,  .48261e+03,  .48622e+03/

  data (v(i,  -3),i=0,9) &
       /  .48985e+03,  .49351e+03,  .49720e+03,  .50091e+03,  .50464e+03, &
       .50840e+03,  .51218e+03,  .51599e+03,  .51982e+03,  .52368e+03/
  
  data (v(i,  -2),i=0,9) &
       /  .52756e+03,  .53147e+03,  .53540e+03,  .53936e+03,  .54335e+03, &
       .54736e+03,  .55140e+03,  .55546e+03,  .55955e+03,  .56367e+03/

  data (v(i,  -1),i=0,9) &
       /  .56782e+03,  .57199e+03,  .57619e+03,  .58042e+03,  .58467e+03, &
       .58895e+03,  .59326e+03,  .59760e+03,  .60196e+03,  .60636e+03/
  
  data (v(i,   0),i=0,9) &
       /  .61078e+03,  .61523e+03,  .61971e+03,  .62422e+03,  .62876e+03, &
       .63332e+03,  .63792e+03,  .64254e+03,  .64720e+03,  .65189e+03/
  
  data (v(i,   1),i=0,9) &
       /  .65660e+03,  .66135e+03,  .66612e+03,  .67093e+03,  .67577e+03, &
       .68063e+03,  .68553e+03,  .69046e+03,  .69542e+03,  .70042e+03/

  data (v(i,   2),i=0,9) &
       /  .70544e+03,  .71050e+03,  .71559e+03,  .72071e+03,  .72586e+03, &
       .73105e+03,  .73627e+03,  .74152e+03,  .74681e+03,  .75212e+03/

  data (v(i,   3),i=0,9) &
       /  .75748e+03,  .76286e+03,  .76828e+03,  .77374e+03,  .77922e+03, &
       .78475e+03,  .79030e+03,  .79589e+03,  .80152e+03,  .80718e+03/

  data (v(i,   4),i=0,9) &
       /  .81288e+03,  .81861e+03,  .82438e+03,  .83019e+03,  .83603e+03, &
       .84190e+03,  .84782e+03,  .85377e+03,  .85975e+03,  .86578e+03/

  data (v(i,   5),i=0,9) &
       /  .87184e+03,  .87794e+03,  .88407e+03,  .89025e+03,  .89646e+03, &
       .90271e+03,  .90900e+03,  .91533e+03,  .92170e+03,  .92810e+03/

  data (v(i,   6),i=0,9) &
       /  .93455e+03,  .94103e+03,  .94756e+03,  .95412e+03,  .96073e+03,&
       .96737e+03,  .97406e+03,  .98078e+03,  .98755e+03,  .99436e+03/
  
  data (v(i,   7),i=0,9) &
       /  .10012e+04,  .10081e+04,  .10150e+04,  .10220e+04,  .10290e+04, &
       .10361e+04,  .10432e+04,  .10503e+04,  .10575e+04,  .10648e+04/
  
  data (v(i,   8),i=0,9) &
       /  .10720e+04,  .10794e+04,  .10867e+04,  .10941e+04,  .11016e+04, &
       .11091e+04,  .11166e+04,  .11242e+04,  .11318e+04,  .11395e+04/

  data (v(i,   9),i=0,9) &
       /  .11472e+04,  .11550e+04,  .11628e+04,  .11707e+04,  .11786e+04, &
       .11866e+04,  .11946e+04,  .12026e+04,  .12107e+04,  .12189e+04/

  data (v(i,  10),i=0,9) &
       /  .12271e+04,  .12353e+04,  .12436e+04,  .12520e+04,  .12604e+04, &
       .12688e+04,  .12773e+04,  .12858e+04,  .12944e+04,  .13031e+04/

  data (v(i,  11),i=0,9) &
       /  .13118e+04,  .13205e+04,  .13293e+04,  .13381e+04,  .13470e+04, &
       .13560e+04,  .13650e+04,  .13741e+04,  .13832e+04,  .13923e+04/

  data (v(i,  12),i=0,9) &
       /  .14015e+04,  .14108e+04,  .14201e+04,  .14295e+04,  .14389e+04, &
       .14484e+04,  .14580e+04,  .14676e+04,  .14772e+04,  .14869e+04/
  
  data (v(i,  13),i=0,9) &
       /  .14967e+04,  .15065e+04,  .15164e+04,  .15263e+04,  .15363e+04, &
       .15464e+04,  .15565e+04,  .15666e+04,  .15769e+04,  .15871e+04/
  
  data (v(i,  14),i=0,9) &
       /  .15975e+04,  .16079e+04,  .16183e+04,  .16289e+04,  .16394e+04, &
       .16501e+04,  .16608e+04,  .16715e+04,  .16824e+04,  .16932e+04/
  
  data (v(i,  15),i=0,9) &
       /  .17042e+04,  .17152e+04,  .17263e+04,  .17374e+04,  .17486e+04, &
       .17599e+04,  .17712e+04,  .17826e+04,  .17940e+04,  .18055e+04/
  
  data (v(i,  16),i=0,9) &
     /  .18171e+04,  .18287e+04,  .18405e+04,  .18522e+04,  .18641e+04, &
     .18760e+04,  .18880e+04,  .19000e+04,  .19121e+04,  .19243e+04/
  
  data (v(i,  17),i=0,9) &
       /  .19365e+04,  .19488e+04,  .19612e+04,  .19737e+04,  .19862e+04, &
       .19988e+04,  .20114e+04,  .20242e+04,  .20370e+04,  .20498e+04/
  
  data (v(i,  18),i=0,9) &
       /  .20628e+04,  .20758e+04,  .20889e+04,  .21020e+04,  .21153e+04, &
       .21286e+04,  .21420e+04,  .21554e+04,  .21689e+04,  .21825e+04/
  
  data (v(i,  19),i=0,9) &
       /  .21962e+04,  .22100e+04,  .22238e+04,  .22377e+04,  .22516e+04, &
       .22657e+04,  .22798e+04,  .22940e+04,  .23083e+04,  .23227e+04/

  data (v(i,  20),i=0,9) &
       /  .23371e+04,  .23516e+04,  .23662e+04,  .23809e+04,  .23957e+04, &
       .24105e+04,  .24254e+04,  .24404e+04,  .24555e+04,  .24706e+04/

  data (v(i,  21),i=0,9) &
       /  .24859e+04,  .25012e+04,  .25166e+04,  .25321e+04,  .25477e+04, &
       .25633e+04,  .25791e+04,  .25949e+04,  .26108e+04,  .26268e+04/

  data (v(i,  22),i=0,9) &
       /  .26429e+04,  .26590e+04,  .26753e+04,  .26916e+04,  .27080e+04, &
       .27245e+04,  .27411e+04,  .27578e+04,  .27746e+04,  .27915e+04/
  
  data (v(i,  23),i=0,9) &
       /  .28084e+04,  .28255e+04,  .28426e+04,  .28598e+04,  .28771e+04, &
       .28946e+04,  .29121e+04,  .29296e+04,  .29473e+04,  .29651e+04/
  
  data (v(i,  24),i=0,9) &
       /  .29830e+04,  .30009e+04,  .30190e+04,  .30372e+04,  .30554e+04, &
       .30738e+04,  .30922e+04,  .31107e+04,  .31294e+04,  .31481e+04/

  data (v(i,  25),i=0,9) &
       /  .31669e+04,  .31859e+04,  .32049e+04,  .32240e+04,  .32432e+04, &
       .32626e+04,  .32820e+04,  .33015e+04,  .33211e+04,  .33409e+04/
  
  data (v(i,  26),i=0,9) &
       /  .33607e+04,  .33806e+04,  .34007e+04,  .34208e+04,  .34411e+04, &
       .34614e+04,  .34819e+04,  .35024e+04,  .35231e+04,  .35438e+04/
  
  data (v(i,  27),i=0,9) &
       /  .35647e+04,  .35857e+04,  .36068e+04,  .36280e+04,  .36493e+04, &
       .36707e+04,  .36922e+04,  .37139e+04,  .37356e+04,  .37575e+04/
  
  data (v(i,  28),i=0,9) &
       /  .37795e+04,  .38015e+04,  .38237e+04,  .38460e+04,  .38684e+04, &
       .38910e+04,  .39136e+04,  .39364e+04,  .39593e+04,  .39822e+04/
  
  data (v(i,  29),i=0,9) &
       /  .40053e+04,  .40286e+04,  .40519e+04,  .40754e+04,  .40989e+04, &
       .41226e+04,  .41464e+04,  .41704e+04,  .41944e+04,  .42186e+04/
  
  data (v(i,  30),i=0,9) &
       /  .42429e+04,  .42673e+04,  .42918e+04,  .43165e+04,  .43413e+04, &
       .43662e+04,  .43912e+04,  .44164e+04,  .44417e+04,  .44671e+04/

  data (v(i,  31),i=0,9) &
       /  .44926e+04,  .45183e+04,  .45440e+04,  .45700e+04,  .45960e+04, &
       .46222e+04,  .46485e+04,  .46749e+04,  .47014e+04,  .47281e+04/

  data (v(i,  32),i=0,9) &
       /  .47550e+04,  .47819e+04,  .48090e+04,  .48362e+04,  .48636e+04, &
       .48911e+04,  .49187e+04,  .49464e+04,  .49743e+04,  .50024e+04/

  data (v(i,  33),i=0,9) &
       /  .50305e+04,  .50588e+04,  .50873e+04,  .51159e+04,  .51446e+04, &
       .51734e+04,  .52024e+04,  .52316e+04,  .52609e+04,  .52903e+04/

  data (v(i,  34),i=0,9) &
       /  .53198e+04,  .53496e+04,  .53794e+04,  .54094e+04,  .54395e+04, &
       .54698e+04,  .55003e+04,  .55308e+04,  .55616e+04,  .55925e+04/

  data (v(i,  35),i=0,9) &
       /  .56235e+04,  .56546e+04,  .56860e+04,  .57174e+04,  .57491e+04, &
       .57808e+04,  .58128e+04,  .58448e+04,  .58771e+04,  .59095e+04/

  data (v(i,  36),i=0,9) &
       /  .59420e+04,  .59747e+04,  .60076e+04,  .60406e+04,  .60737e+04, &
       .61071e+04,  .61405e+04,  .61742e+04,  .62080e+04,  .62419e+04/
  
  data (v(i,  37),i=0,9) &
       /  .62761e+04,  .63103e+04,  .63448e+04,  .63794e+04,  .64141e+04, &
       .64491e+04,  .64842e+04,  .65194e+04,  .65549e+04,  .65905e+04/

  data (v(i,  38),i=0,9) &
       /  .66262e+04,  .66622e+04,  .66982e+04,  .67345e+04,  .67709e+04, &
       .68076e+04,  .68443e+04,  .68813e+04,  .69184e+04,  .69557e+04/

  data (v(i,  39),i=0,9) &
       /  .69932e+04,  .70308e+04,  .70686e+04,  .71066e+04,  .71448e+04, &
       .71831e+04,  .72217e+04,  .72604e+04,  .72992e+04,  .73383e+04/
  
  data (v(i,  40),i=0,9) &
       /  .73775e+04,  .74170e+04,  .74566e+04,  .74964e+04,  .75363e+04, &
       .75765e+04,  .76168e+04,  .76573e+04,  .76981e+04,  .77389e+04/
  
  data (v(i,  41),i=0,9) &
       /  .77800e+04,  .78213e+04,  .78628e+04,  .79044e+04,  .79463e+04, &
       .79883e+04,  .80305e+04,  .80729e+04,  .81155e+04,  .81583e+04/

  data (v(i,  42),i=0,9) &
       /  .82013e+04,  .82445e+04,  .82879e+04,  .83315e+04,  .83753e+04, &
       .84193e+04,  .84635e+04,  .85078e+04,  .85524e+04,  .85972e+04/

  data (v(i,  43),i=0,9) &
       /  .86422e+04,  .86874e+04,  .87328e+04,  .87784e+04,  .88242e+04, &
       .88702e+04,  .89164e+04,  .89628e+04,  .90094e+04,  .90563e+04/
  
  data (v(i,  44),i=0,9) &
       /  .91033e+04,  .91506e+04,  .91981e+04,  .92457e+04,  .92936e+04, &
       .93418e+04,  .93901e+04,  .94386e+04,  .94874e+04,  .95363e+04/

  data (v(i,  45),i=0,9) &
       /  .95855e+04,  .96349e+04,  .96846e+04,  .97344e+04,  .97845e+04, &
       .98348e+04,  .98853e+04,  .99360e+04,  .99870e+04,  .10038e+05/
  
  data (v(i,  46),i=0,9) &
       /  .10090e+05,  .10141e+05,  .10193e+05,  .10245e+05,  .10297e+05, &
       .10350e+05,  .10403e+05,  .10456e+05,  .10509e+05,  .10563e+05/
  
  data (v(i,  47),i=0,9) &
       /  .10616e+05,  .10670e+05,  .10724e+05,  .10779e+05,  .10833e+05, &
       .10888e+05,  .10944e+05,  .10999e+05,  .11055e+05,  .11110e+05/
  
  data (v(i,  48),i=0,9) &
       /  .11166e+05,  .11223e+05,  .11279e+05,  .11336e+05,  .11393e+05, &
       .11451e+05,  .11508e+05,  .11566e+05,  .11624e+05,  .11682e+05/

  data (v(i,  49),i=0,9) &
       /  .11741e+05,  .11800e+05,  .11859e+05,  .11918e+05,  .11978e+05, &
       .12038e+05,  .12098e+05,  .12158e+05,  .12219e+05,  .12280e+05/
  
  data (v(i,  50),i=0,9) &
       /  .12341e+05,  .12402e+05,  .12464e+05,  .12526e+05,  .12588e+05, &
       .12650e+05,  .12713e+05,  .12776e+05,  .12839e+05,  .12903e+05/
  
  data (v(i,  51),i=0,9) &
       /  .12967e+05,  .13031e+05,  .13095e+05,  .13160e+05,  .13225e+05, &
       .13290e+05,  .13355e+05,  .13421e+05,  .13487e+05,  .13553e+05/
  
  data (v(i,  52),i=0,9) &
       /  .13620e+05,  .13687e+05,  .13754e+05,  .13821e+05,  .13889e+05, &
       .13957e+05,  .14025e+05,  .14093e+05,  .14162e+05,  .14231e+05/

  data (v(i,  53),i=0,9) &
       /  .14301e+05,  .14370e+05,  .14440e+05,  .14511e+05,  .14581e+05, &
       .14652e+05,  .14723e+05,  .14795e+05,  .14866e+05,  .14938e+05/

  data (v(i,  54),i=0,9) &
       /  .15011e+05,  .15083e+05,  .15156e+05,  .15230e+05,  .15303e+05, &
       .15377e+05,  .15451e+05,  .15526e+05,  .15600e+05,  .15675e+05/

  data (v(i,  55),i=0,9) &
       /  .15751e+05,  .15827e+05,  .15903e+05,  .15979e+05,  .16055e+05, &
       .16132e+05,  .16210e+05,  .16287e+05,  .16365e+05,  .16443e+05/

  data (v(i,  56),i=0,9) &
       /  .16522e+05,  .16601e+05,  .16680e+05,  .16759e+05,  .16839e+05, &
       .16919e+05,  .17000e+05,  .17081e+05,  .17162e+05,  .17243e+05/
  
  data (v(i,  57),i=0,9) &
       /  .17325e+05,  .17407e+05,  .17490e+05,  .17573e+05,  .17656e+05, &
       .17739e+05,  .17823e+05,  .17907e+05,  .17992e+05,  .18076e+05/

  data (v(i,  58),i=0,9) &
       /  .18162e+05,  .18247e+05,  .18333e+05,  .18419e+05,  .18506e+05, &
       .18593e+05,  .18680e+05,  .18768e+05,  .18856e+05,  .18944e+05/
  
  data (v(i,  59),i=0,9) &
       /  .19032e+05,  .19122e+05,  .19211e+05,  .19301e+05,  .19391e+05, &
       .19481e+05,  .19572e+05,  .19663e+05,  .19755e+05,  .19847e+05/
  
  data (v(i,  60),i=0,9) &
       /  .19939e+05,  .20031e+05,  .20124e+05,  .20218e+05,  .20312e+05, &
       .20406e+05,  .20500e+05,  .20595e+05,  .20690e+05,  .20786e+05/
  
  data (v(i,  61),i=0,9) &
       /  .20882e+05,  .20978e+05,  .21075e+05,  .21172e+05,  .21270e+05, &
       .21367e+05,  .21466e+05,  .21564e+05,  .21663e+05,  .21763e+05/
  
  data (v(i,  62),i=0,9) &
       /  .21863e+05,  .21963e+05,  .22063e+05,  .22164e+05,  .22266e+05, &
       .22368e+05,  .22470e+05,  .22572e+05,  .22675e+05,  .22779e+05/
  
  data (v(i,  63),i=0,9) &
       /  .22883e+05,  .22987e+05,  .23091e+05,  .23196e+05,  .23302e+05, &
        .23408e+05,  .23514e+05,  .23621e+05,  .23728e+05,  .23835e+05/
  
  data (v(i,  64),i=0,9) &
       /  .23943e+05,  .24051e+05,  .24160e+05,  .24269e+05,  .24379e+05, &
       .24489e+05,  .24599e+05,  .24710e+05,  .24821e+05,  .24933e+05/
  
  data (v(i,  65),i=0,9) &
       /  .25045e+05,  .25158e+05,  .25271e+05,  .25384e+05,  .25498e+05, &
       .25612e+05,  .25727e+05,  .25842e+05,  .25958e+05,  .26074e+05/
  
  data (v(i,  66),i=0,9) &
       /  .26190e+05,  .26307e+05,  .26424e+05,  .26542e+05,  .26661e+05, &
       .26779e+05,  .26898e+05,  .27018e+05,  .27138e+05,  .27259e+05/

  data (v(i,  67),i=0,9) &
       /  .27380e+05,  .27501e+05,  .27623e+05,  .27745e+05,  .27868e+05, &
       .27991e+05,  .28115e+05,  .28239e+05,  .28364e+05,  .28489e+05/

  data (v(i,  68),i=0,9) &
       /  .28615e+05,  .28741e+05,  .28868e+05,  .28995e+05,  .29122e+05, &
       .29250e+05,  .29379e+05,  .29508e+05,  .29637e+05,  .29767e+05/

  data (v(i,  69),i=0,9) &
       /  .29897e+05,  .30028e+05,  .30160e+05,  .30292e+05,  .30424e+05, &
       .30557e+05,  .30690e+05,  .30824e+05,  .30958e+05,  .31093e+05/

  data (v(i,  70),i=0,9) &
       /  .31229e+05,  .31364e+05,  .31501e+05,  .31638e+05,  .31775e+05, &
       .31913e+05,  .32051e+05,  .32190e+05,  .32330e+05,  .32470e+05/

  data (v(i,  71),i=0,9) &
       /  .32610e+05,  .32751e+05,  .32892e+05,  .33034e+05,  .33177e+05, &
       .33320e+05,  .33464e+05,  .33608e+05,  .33752e+05,  .33897e+05/

  data (v(i,  72),i=0,9) &
       /  .34043e+05,  .34189e+05,  .34336e+05,  .34483e+05,  .34631e+05, &
       .34779e+05,  .34928e+05,  .35078e+05,  .35228e+05,  .35378e+05/

  data (v(i,  73),i=0,9) &
       /  .35529e+05,  .35681e+05,  .35833e+05,  .35986e+05,  .36139e+05, &
       .36293e+05,  .36448e+05,  .36603e+05,  .36758e+05,  .36914e+05/

  data (v(i,  74),i=0,9) &
       /  .37071e+05,  .37228e+05,  .37386e+05,  .37544e+05,  .37703e+05, &
       .37863e+05,  .38023e+05,  .38183e+05,  .38344e+05,  .38506e+05/

  data (v(i,  75),i=0,9) &
       /  .38669e+05,  .38832e+05,  .38995e+05,  .39159e+05,  .39324e+05, &
       .39489e+05,  .39655e+05,  .39822e+05,  .39989e+05,  .40156e+05/

  data (v(i,  76),i=0,9) &
       /  .40325e+05,  .40494e+05,  .40663e+05,  .40833e+05,  .41004e+05, &
       .41175e+05,  .41347e+05,  .41520e+05,  .41693e+05,  .41866e+05/

  data (v(i,  77),i=0,9) &
       /  .42041e+05,  .42216e+05,  .42391e+05,  .42568e+05,  .42744e+05, &
       .42922e+05,  .43100e+05,  .43279e+05,  .43458e+05,  .43638e+05/
  
  data (v(i,  78),i=0,9) &
       /  .43819e+05,  .44000e+05,  .44182e+05,  .44364e+05,  .44547e+05, &
       .44731e+05,  .44916e+05,  .45101e+05,  .45286e+05,  .45473e+05/
  
  data (v(i,  79),i=0,9) &
       /  .45660e+05,  .45848e+05,  .46036e+05,  .46225e+05,  .46415e+05, &
       .46605e+05,  .46796e+05,  .46988e+05,  .47180e+05,  .47373e+05/
  
  data (v(i,  80),i=0,9) &
       /  .47567e+05,  .47761e+05,  .47956e+05,  .48152e+05,  .48348e+05, &
       .48545e+05,  .48743e+05,  .48941e+05,  .49140e+05,  .49340e+05/
  
  data (v(i,  81),i=0,9) &
       /  .49540e+05,  .49742e+05,  .49943e+05,  .50146e+05,  .50349e+05, &
       .50553e+05,  .50758e+05,  .50963e+05,  .51169e+05,  .51376e+05/

  data (v(i,  82),i=0,9) &
       /  .51583e+05,  .51792e+05,  .52001e+05,  .52210e+05,  .52420e+05, &
       .52632e+05,  .52843e+05,  .53056e+05,  .53269e+05,  .53483e+05/

  data (v(i,  83),i=0,9) &
       /  .53698e+05,  .53913e+05,  .54129e+05,  .54346e+05,  .54564e+05, &
       .54782e+05,  .55001e+05,  .55221e+05,  .55441e+05,  .55663e+05/

  data (v(i,  84),i=0,9) &
       /  .55885e+05,  .56108e+05,  .56331e+05,  .56556e+05,  .56781e+05, &
       .57007e+05,  .57233e+05,  .57461e+05,  .57689e+05,  .57918e+05/

  data (v(i,  85),i=0,9) &
       /  .58147e+05,  .58378e+05,  .58609e+05,  .58841e+05,  .59074e+05, &
       .59307e+05,  .59542e+05,  .59777e+05,  .60013e+05,  .60250e+05/

  data (v(i,  86),i=0,9) &
       /  .60487e+05,  .60725e+05,  .60964e+05,  .61204e+05,  .61445e+05, &
       .61687e+05,  .61929e+05,  .62172e+05,  .62416e+05,  .62661e+05/
  
  data (v(i,  87),i=0,9) &
       /  .62906e+05,  .63152e+05,  .63400e+05,  .63648e+05,  .63896e+05, &
       .64146e+05,  .64397e+05,  .64648e+05,  .64900e+05,  .65153e+05/
  
  data (v(i,  88),i=0,9) &
       /  .65407e+05,  .65661e+05,  .65917e+05,  .66173e+05,  .66430e+05, &
       .66688e+05,  .66947e+05,  .67207e+05,  .67468e+05,  .67729e+05/
  
  data (v(i,  89),i=0,9) &
       /  .67991e+05,  .68254e+05,  .68518e+05,  .68783e+05,  .69049e+05, &
       .69316e+05,  .69583e+05,  .69851e+05,  .70121e+05,  .70391e+05/

  data (v(i,  90),i=0,9) &
       /  .70662e+05,  .70934e+05,  .71206e+05,  .71480e+05,  .71755e+05, &
       .72030e+05,  .72306e+05,  .72583e+05,  .72862e+05,  .73141e+05/

  data (v(i,  91),i=0,9) &
       /  .73421e+05,  .73701e+05,  .73983e+05,  .74266e+05,  .74549e+05, &
       .74834e+05,  .75119e+05,  .75406e+05,  .75693e+05,  .75981e+05/

  data (v(i,  92),i=0,9) &
       /  .76270e+05,  .76560e+05,  .76851e+05,  .77143e+05,  .77436e+05, &
       .77730e+05,  .78025e+05,  .78320e+05,  .78617e+05,  .78914e+05/
  
  data (v(i,  93),i=0,9) &
       /  .79213e+05,  .79512e+05,  .79813e+05,  .80114e+05,  .80417e+05, &
       .80720e+05,  .81024e+05,  .81330e+05,  .81636e+05,  .81943e+05/
  
  data (v(i,  94),i=0,9) &
       /  .82251e+05,  .82561e+05,  .82871e+05,  .83182e+05,  .83494e+05, &
       .83807e+05,  .84121e+05,  .84436e+05,  .84753e+05,  .85070e+05/

  data (v(i,  95),i=0,9) &
       /  .85388e+05,  .85707e+05,  .86027e+05,  .86348e+05,  .86670e+05, &
       .86994e+05,  .87318e+05,  .87643e+05,  .87969e+05,  .88297e+05/

  data (v(i,  96),i=0,9) &
       /  .88625e+05,  .88954e+05,  .89285e+05,  .89616e+05,  .89949e+05, &
       .90282e+05,  .90617e+05,  .90952e+05,  .91289e+05,  .91627e+05/
  
  data (v(i,  97),i=0,9) &
       /  .91965e+05,  .92305e+05,  .92646e+05,  .92988e+05,  .93331e+05, &
       .93675e+05,  .94020e+05,  .94367e+05,  .94714e+05,  .95062e+05/

  data (v(i,  98),i=0,9) &
       /  .95412e+05,  .95762e+05,  .96114e+05,  .96467e+05,  .96821e+05, &
       .97176e+05,  .97532e+05,  .97889e+05,  .98247e+05,  .98606e+05/
  
  data (v(i,  99),i=0,9) &
       /  .98967e+05,  .99328e+05,  .99691e+05,  .10005e+06,  .10042e+06, &
       .10079e+06,  .10115e+06,  .10152e+06,  .10189e+06,  .10226e+06/
  
  data (v(i, 100),i=0,9) &
       /  .10263e+06,  .10301e+06,  .10338e+06,  .10376e+06,  .10413e+06, &
       .10451e+06,  .10489e+06,  .10527e+06,  .10565e+06,  .10603e+06/
  
  
!     ------------------------------------------------------------------
!     convert temp & dew point to celcius
!     ------------------------------------------------------------------

  t(1) = temp - 273.16
  t(2) = dpt - 273.16

!     ------------------------------------------------------------------
!     if temps are within reasonable range. 
!     ------------------------------------------------------------------

  if( (t(1).gt.-50.0).and.(t(2).gt.-100.0) )then

!     ------------------------------------------------------------------
!       loop thru the two temperatures.
!     ------------------------------------------------------------------

     do k = 1, 2

!     ------------------------------------------------------------------
!         derive look-up table indices from the temp value. and get
!         vapor presure from the common block lookup table array. vapor
!         pressure will be saturation value for k=1 and ambient value
!         for k=2.
!     ------------------------------------------------------------------

        it   = int(t(k))
        itf  = int(mod(abs(t(k))*10.0,10.0))
        e(k) = v(itf,it)
     enddo

!     ------------------------------------------------------------------
!       calculate relative humidity from saturation and ambient vapor
!       pressure.
!     ------------------------------------------------------------------

     AGRMET_calcrh_dpt = min(max((e(2) / e(1)),0.0),1.0)
  else

!     ------------------------------------------------------------------
!       if temps are not in reasonable range, set rh to -1.0.
!     ------------------------------------------------------------------

     AGRMET_calcrh_dpt = -1.0
     
  endif
  
  return
end function AGRMET_calcrh_dpt
