!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center
! Land Information System Framework (LISF)
! Version 7.4
!
! Copyright (c) 2022 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
! 
! !MODULE: GlobSnow_obsMod
! \label(GlobSnow_obsMod)
!
! !INTERFACE:
module GlobSnow_obsMod
! 
! !USES:   
  use ESMF

  implicit none

  PRIVATE 
!
! !INPUT PARAMETERS: 
! 
! !OUTPUT PARAMETERS:
!
! !DESCRIPTION: 
!  This module handles the observation plugin for the GlobSnow snow
!  products which include snow water equivalent (SWE) and snow extent (SE). 
!  The product is generated by assimilation of satellite data with 
!  in situ observations-derived background on snow depth. 
!  
!  Snow extent is based on optical ERS-2 ATSR (1995-) and ENVISAT AATSR (2002-)
!  SWE is based on AMSR-E (2002-present), SSM/I (1987-2002), SNMR 
!  data (1978-1987) and WMO Global weather station data. SWE algorithm 
!  by Pulliainen (2006)
!   
!   Pulliainen, J. "Mapping of snow water equivalent and snow depth
!   in boreal and sub-arctic zones by assimilating space-borne 
!   microwave radiometer data and ground-based observations", 
!   Remote Sensing of Environment, 101 (2), 2006, 257-269. 
!  
!  http://www.globsnow.info/index.php?page=Data
! 
! !FILES USED:
!
! !REVISION HISTORY: 
!  10 Dec 2010   Sujay Kumar  Initial Specification
! 
!EOP
!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!-----------------------------------------------------------------------------
  PUBLIC :: GlobSnow_obsinit !Initializes structures for reading GlobSnow data
!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------
  PUBLIC :: GlobSnowobs !Object to hold GlobSnow observation attributes
!EOP

  type, public :: globsnowdec
     character*100         :: odir
     real, allocatable           :: rlat(:)
     real, allocatable           :: rlon(:)
     integer, allocatable        :: n11(:)
  end type globsnowdec
     
  type(globsnowdec), allocatable :: GlobSnowObs(:)

contains
  
!BOP
! 
! !ROUTINE: GlobSnow_obsInit
! \label{GlobSnow_obsInit}
!
! !INTERFACE: 
  subroutine GlobSnow_obsinit(i)
! 
! !USES: 
    use LVT_coreMod,   only : LVT_rc, LVT_Config
    use LVT_histDataMod
    use LVT_logMod,    only : LVT_verify
    use LVT_timeMgrMod

    implicit none
!
! !INPUT PARAMETERS: 
    integer,   intent(IN) :: i 
! 
! !OUTPUT PARAMETERS:
!
! !DESCRIPTION: 
!   This subroutine initializes and sets up the data structures required
!   for reading the GlobSnow data, including the computation of spatial 
!   interpolation weights. The GlobSnow data is provides in the 
!   EASE grid projection. 
! 
! !FILES USED:
!
! !REVISION HISTORY: 
! 
!EOP
    integer               :: status
    real                  :: gridDesci(50)

    if(.not.allocated(GlobSnowObs)) then 
       allocate(GlobSnowObs(LVT_rc%nDataStreams))
    endif

    call ESMF_ConfigGetAttribute(LVT_Config, GlobSnowObs(i)%odir, &
         label='GlobSnow data directory: ',rc=status)
    call LVT_verify(status, 'GlobSnow data directory: not defined')

    call LVT_update_timestep(LVT_rc, 86400)

    allocate(GlobSnowObs(i)%rlat(LVT_rc%lnc*LVT_rc%lnr))
    allocate(GlobSnowObs(i)%rlon(LVT_rc%lnc*LVT_rc%lnr))

    allocate(GlobSnowObs(i)%n11(LVT_rc%lnc*LVT_rc%lnr))

    gridDesci = 0
    
    !filling the items needed by the interpolation library
    gridDesci(1) = 9  !input is EASE grid
    gridDesci(2) = 721
    gridDesci(3) = 721
    gridDesci(9) = 2   !Nl
 
    call neighbor_interp_input(gridDesci,LVT_rc%gridDesc,&
         LVT_rc%lnc*LVT_rc%lnr, &
         GlobSnowObs(i)%rlat, GlobSnowObs(i)%rlon,&
         GlobSnowObs(i)%n11)


  end subroutine GlobSnow_obsinit


end module GlobSnow_obsMod
